
autosteer = {};

function autosteer.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function autosteer:load(xmlFile)
	self.deactivateAS = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.autoSteer#deactivate"), false);
	if not self.deactivateAS then
		self.setAutoSteer = SpecializationUtil.callSpecializationsFunction("setAutoSteer");
		
		self.oldAutoRotateBackSpeed = self.autoRotateBackSpeed;
		self.oldMaxRotatedTimeSpeed = self.maxRotatedTimeSpeed;
		self.haveAutoSteer = self;
		
		self.steer = {};
		self.steer.invertAxis = Utils.getNoNil(getXMLBool(xmlFile,"vehicle.autoSteer#invertAxis"), false);
		self.steer.speed = Utils.getNoNil(getXMLFloat(xmlFile,"vehicle.autoSteer#steeringSpeed"), 1000);
		
		local activated = Utils.getNoNil(getXMLBool(xmlFile,"vehicle.autoSteer#activated"), false);
		if activated then
			self.steeringLocked = false;
			self.autoRotateBackSpeed = 0;
			self.maxRotatedTimeSpeed = 0;
		else
			self.steeringLocked = true;
		end;
		
		self.showAutoSteerHud = Utils.getNoNil(getXMLBool(xmlFile,"vehicle.autoSteer#hud"), true);
		self.isSelectable = true;
		self.overrideUpdateVehicle = true;
		
		self:addConflictCheckedInput(InputBinding.AUTOSTEER_1);
	end;
end;

function autosteer:delete()
end;

function autosteer:readStream(streamId, connection)
	if not self.deactivateAS then
		self:setAutoSteer(streamReadBool(streamId), true);
	end;
end;

function autosteer:writeStream(streamId, connection)
    if not self.deactivateAS then
		streamWriteBool(streamId, self.steeringLocked);
	end;
end;

function autosteer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function autosteer:keyEvent(unicode, sym, modifier, isDown)
end;

function autosteer:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not self.deactivateAS then
		if not resetVehicles then
			local autoSteer = Utils.getNoNil(getXMLBool(xmlFile, key .. "#autoSteer"), true);
			self:setAutoSteer(autoSteer);
		end;
		
		return BaseMission.VEHICLE_LOAD_OK;
	end;
end;

function autosteer:getSaveAttributesAndNodes(nodeIdent)
	if not self.deactivateAS then
		local attributes = 'autoSteer="' .. tostring(self.steeringLocked) .. '"';
		return attributes, nil;
	end;
end;

function autosteer:update(dt)
	if not self.deactivateAS then
		if self:getIsActive() and self.isClient then
			if self:getIsActiveForInput() and not self:hasInputConflictWithSelection() then
				if InputBinding.hasEvent(InputBinding.AUTOSTEER_1) then
					self:setAutoSteer(not self.steeringLocked);
				end;
			end;
		end;
	end;
end;

function autosteer:updateTick(dt)
end;

function autosteer:draw()
	if not self.deactivateAS then
		if self.isClient then
			if self:getIsActiveForInput() then
				if g_currentMission.showHelpText then
					if self.steeringLocked then
						g_currentMission:addHelpButtonText(g_i18n:getText("AUTOSTEER_4"), InputBinding.AUTOSTEER_1);
					else
						g_currentMission:addHelpButtonText(g_i18n:getText("AUTOSTEER_5"), InputBinding.AUTOSTEER_1);
					end;
				end;
				
				if g_currentMission.autoSteerHud ~= nil then
					if self.showAutoSteerHud then
						g_currentMission.autoSteerHud:render();
						local text = g_i18n:getText("AUTOSTEER_3");
						if not self.steeringLocked then
							text = g_i18n:getText("AUTOSTEER_2");
						end;
						
						setTextBold(true);
						setTextAlignment(RenderText.ALIGN_CENTER);
						
						setTextColor(0, 0, 0, 1);
						renderText(g_currentMission.autoSteerHud.x + 0.147, g_currentMission.autoSteerHud.y + 0.010, 0.018, text);
						setTextColor(1, 1, 1, 1);
						renderText(g_currentMission.autoSteerHud.x + 0.147, g_currentMission.autoSteerHud.y + 0.012, 0.018, text);
						
						setTextAlignment(RenderText.ALIGN_LEFT);
						setTextBold(false);
					end;
				end;  
			end;  
		end;
	end;  
end;

function autosteer:setAutoSteer(lockedsteer, noEventSend)
	if lockedsteer ~= self.steeringLocked then
		SetAutoSteerEvent.sendEvent(self, lockedsteer, noEventSend)
		self.steeringLocked = lockedsteer;
		
		if not lockedsteer then
			self.autoRotateBackSpeed = 0;
			self.maxRotatedTimeSpeed = 0;
		else
			self.autoRotateBackSpeed = self.oldAutoRotateBackSpeed;
			self.maxRotatedTimeSpeed = self.oldMaxRotatedTimeSpeed;
		end;
	end;
end;

-- Event --
SetAutoSteerEvent = {};
SetAutoSteerEvent_mt = Class(SetAutoSteerEvent, Event);
  
InitEventClass(SetAutoSteerEvent, "SetAutoSteerEvent");
  
function SetAutoSteerEvent:emptyNew()
	local self = Event:new(SetAutoSteerEvent_mt);
	self.className="SetAutoSteerEvent";
	return self;
end;
  
function SetAutoSteerEvent:new(object, lockedsteer)
	local self = SetAutoSteerEvent:emptyNew()
	self.object = object;
	self.lockedsteer = lockedsteer;
	return self;
end;
  
function SetAutoSteerEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.lockedsteer = streamReadBool(streamId);
	self.object = networkGetObject(id);
	self:run(connection);
end;
  
function SetAutoSteerEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.lockedsteer);
end;
  
function SetAutoSteerEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(self, false, connection, self.object);
	end;
	self.object:setAutoSteer(self.lockedsteer, true);
end;
  
function SetAutoSteerEvent.sendEvent(vehicle, lockedsteer, noEventSend)
	if lockedsteer ~= vehicle.steeringLocked then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(SetAutoSteerEvent:new(vehicle, lockedsteer), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(SetAutoSteerEvent:new(vehicle, lockedsteer));
			end;
		end;
	end;
end;