﻿
wheelOverride = {};

wheelOverride.updateVehiclePhysics = function(self, oldFunc, axisForward, axisForwardIsAnalog, axisSide, axisSideIsAnalog, dt)
	if self.haveAutoSteer ~= nil and self.haveAutoSteer == self and self.overrideUpdateVehicle then
		local acceleration = 0;
		if self.isMotorStarted and self.motorStartTime <= self.time then
			acceleration = -axisForward;
			if math.abs(acceleration) > 0.8 then
				if self.motor.speedLevel ~= 0 then
					self.lastAcceleration = self.lastAcceleration*0.5;
				end;
				self.motor:setSpeedLevel(0, true)
			end;
			if self.motor.speedLevel ~= 0 then
				acceleration = self.motor.accelerations[self.motor.speedLevel];
			end;
		end;
		if self.fuelFillLevel == 0 then
			acceleration = 0;
		end;
		
		-- Check if the shuttle is enabled --
		if not self.shuttleOverride and self.shuttleEnabled and self.shuttleEnabled ~= nil then
			if acceleration > 0 then
				acceleration = acceleration * self.shuttleDirection;
			elseif self.movingDirection <= 0 then
				acceleration = 0;
			end;
		end;
		
		acceleration = Steerable.calculateRealAcceleration(self, acceleration, dt);
	  
		local inputAxisX = axisSide;
		if self.haveAutoSteer ~= nil and self.steer.invertAxis ~= nil and self.steer.invertAxis then
			inputAxisX = -inputAxisX;
		end;
		local steeringSpeed = 1000;
		if self.steer ~= nil and self.steer.speed ~= nil then
			steeringSpeed = self.steer.speed;
		end;	
		
		if axisSideIsAnalog then
			if self.steeringLocked ~= nil and not self.steeringLocked then
				if inputAxisX < 0 then
					self.rotatedTime = math.min(self.rotatedTime - dt / steeringSpeed * inputAxisX, self.maxRotTime);
				elseif inputAxisX > 0 then
					self.rotatedTime = math.max(self.rotatedTime - dt / steeringSpeed * inputAxisX, self.minRotTime);
				end;
			else
				local targetRotatedTime = 0;
				if inputAxisX < 0 then
					targetRotatedTime = math.min(-self.maxRotTime * inputAxisX, self.maxRotTime);
				else
					targetRotatedTime = math.max(self.minRotTime * inputAxisX, self.minRotTime);
				end;
			  
				local maxTime = self.maxRotatedTimeSpeed*dt;
				if math.abs(targetRotatedTime-self.rotatedTime) > maxTime then
					if targetRotatedTime > self.rotatedTime then
						self.rotatedTime = self.rotatedTime + maxTime;
					else
						self.rotatedTime = self.rotatedTime - maxTime;
					end;
				else
					self.rotatedTime = targetRotatedTime;
				end;
			end;
		else 
			local rotScale = math.min(1.0 / (self.lastSpeed * self.speedRotScale + self.speedRotScaleOffset), 1);
			if inputAxisX < 0 then
				self.rotatedTime = math.min(self.rotatedTime - dt / steeringSpeed * inputAxisX * rotScale, self.maxRotTime);
			elseif inputAxisX > 0 then
				self.rotatedTime = math.max(self.rotatedTime - dt / steeringSpeed * inputAxisX * rotScale, self.minRotTime);
			else
				if self.autoRotateBackSpeed ~= 0 then
					if self.rotatedTime > 0 then
						self.rotatedTime = math.max(self.rotatedTime - dt * 0.001 * self.autoRotateBackSpeed * rotScale, 0);
					else
						self.rotatedTime = math.min(self.rotatedTime + dt * 0.001 * self.autoRotateBackSpeed * rotScale, 0);
					end;
				end;
			end;
		end;
		
		if self.firstTimeRun then
			self.motor.rpmLimit = self:getRpmLimit();
			WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeed, acceleration, false, self.requiredDriveMode)
		end;
	else
		oldFunc(self, axisForward, axisForwardIsAnalog, axisSide, axisSideIsAnalog, dt);
	end;
end;
Steerable.updateVehiclePhysics = Utils.overwrittenFunction(Steerable.updateVehiclePhysics, wheelOverride.updateVehiclePhysics);